From: Alasdair G Kergon <agk@redhat.com>

Add a target feature flag to indicate that a target does not work in
read-only mode.

The thin provisioning target uses this.

Signed-off-by: Alasdair G Kergon <agk@redhat.com>
---
 drivers/md/dm-table.c         |    6 ++++++
 include/linux/device-mapper.h |    4 ++++
 2 files changed, 10 insertions(+)

Index: linux-3.0/drivers/md/dm-table.c
===================================================================
--- linux-3.0.orig/drivers/md/dm-table.c
+++ linux-3.0/drivers/md/dm-table.c
@@ -774,6 +774,12 @@ int dm_table_add_target(struct dm_table 
 		t->singleton = 1;
 	}
 
+	if (dm_target_always_writeable(tgt->type) && !(t->mode & FMODE_WRITE)) {
+		DMERR("%s: target type %s may not be included in read-only tables",
+		      dm_device_name(t->md), type);
+		return -EINVAL;
+	}
+
 	tgt->table = t;
 	tgt->begin = start;
 	tgt->len = len;
Index: linux-3.0/include/linux/device-mapper.h
===================================================================
--- linux-3.0.orig/include/linux/device-mapper.h
+++ linux-3.0/include/linux/device-mapper.h
@@ -165,6 +165,10 @@ struct target_type {
 #define DM_TARGET_SINGLETON		0x00000001
 #define dm_target_needs_singleton(type)	((type)->features & DM_TARGET_SINGLETON)
 
+#define DM_TARGET_ALWAYS_WRITEABLE	0x00000002
+#define dm_target_always_writeable(type) \
+		((type)->features & DM_TARGET_ALWAYS_WRITEABLE)
+
 struct dm_target {
 	struct dm_table *table;
 	struct target_type *type;
